@@ -38,6 +38,15 @@ namespace xcas {
 #endif // ndef NO_NAMESPACE_XCAS
 
 #ifdef HAVE_LIBFLTK
+  const int QRDISP_WIDTH=185,QRDISP_HEIGHT=185;
+  class QRGraph:public Fl_Widget {
+  public:
+    int QRscale,size_border;
+    string filename;
+    unsigned char data[QRDISP_WIDTH][QRDISP_HEIGHT];
+    virtual FL_EXPORT void draw();
+    QRGraph(int x,int y,int scale=3,const char * f="qrcode"):QRscale(scale),Fl_Widget(x,y,200*scale,200*scale),filename(f){};
+  };
   bool QRdisp(const char * text);
 
   // Locales initialization, should be done before history.o is loaded
@@ -127,7 +136,7 @@ namespace xcas {
     int mode; // 0 pointer, 1 1-arg, 2 2-args, etc.
     // plot_tmp=function_tmp(args_tmp) or function_final(args_tmp)
     // depends whether args.tmp.size()==mode
-    giac::gen function_tmp,function_final,args_push; 
+    giac::gen function_tmp,function_final,args_push,trace_x0,trace_x1,trace_x2,trace_y0,trace_y1,trace_y2,trace_parameq,trace_z1,trace_z2,trace_t; // trace_* are storing cached derivatives for curve study; 
     giac::vecteur args_tmp; // WARNING should only contain numeric value
     unsigned args_tmp_push_size;
   public:
@@ -135,6 +144,7 @@ namespace xcas {
     std::vector<std::string> args_help;
     bool no_handle; // disable mouse handling
     bool show_mouse_on_object; // FL_MOVE always handled or not
+    bool resize_ortho;
     unsigned display_mode ; 
     // bit0=1 plot_instructions, bit1=1 animations_instruction
     // bit2=1 glFrustum/glOrtho, bit3=1 GL_LIGHTING, bit4=1 GL_FLAT, 
@@ -291,7 +301,7 @@ namespace xcas {
   void find_dxdy(const std::string & legendes,int labelpos,int labelsize,int & dx,int & dy);
   std::string print_color(int couleur);
 
-  int figure_param_dialog(Fl_Widget * f,bool adjust,double & tmin,double & tmax,double & tcurrent,double & tstep,std::string & name,bool symb,std::string & tmp);
+  int figure_param_dialog(Fl_Widget * f,bool adjust,double & tmin,double & tmax,double & tcurrent,double & tstep,std::string & name,bool symb,std::string & tmp,bool ispnt);
 
   giac::gen geometry_round_numeric(double x,double y,double eps,bool approx);
   giac::gen geometry_round_numeric(double x,double y,double z,double eps,bool approx);
@@ -393,6 +403,7 @@ namespace xcas {
     Editeur * ed;
     Fl_Menu_Bar * menubar ;
     int scroll_position;
+    int disposition;
     Logo(int X,int Y,int W,int H,int L);
     virtual void resize(int X,int Y,int W,int H);
     virtual void draw();
